@@ -2,45 +2,69 @@ package WebService::NetSuite::SuiteTalk;
 
 use Modern::Perl '2011';    ## no critic (Modules::ProhibitUseQuotedVersion)
 
-our $VERSION = '0.006';     # VERSION
+our $VERSION = '0.007';     # VERSION
 use utf8;
 ## no critic (Subroutines::ProhibitCallsToUndeclaredSubs)
 use List::Util 1.33 'any';
 use Moo;
-use MooX::Types::MooseLike::Base qw(CodeRef HashRef);
 use POSIX 'floor';
 with 'WebService::NetSuite::SuiteTalk::Role::Connection';
 
-has _calls => ( is => 'ro', isa => HashRef [CodeRef], default => sub { {} } );
-
-sub _call {
-    my ( $self, $operation_name, $parameters_ref, @call_options ) = @_;
-    $self->_operation_name($operation_name);
-
-    # cache compiled client code
-    if ( not exists $self->_calls->{$operation_name} ) {
-        $self->_calls->{$operation_name}
-            = $self->wsdl->compileClient( $operation_name,
-            transport => $self->transport );
+sub BUILD {
+    my $self = shift;
+    for my $method ( map { $_->name } $self->wsdl->operations ) {
+        ## no critic (ProhibitNoStrict,ProhibitNoWarnings)
+        no strict 'refs';
+        no warnings 'redefine';
+        *{$method} = _method_closure($method);
+        around $method => _around_closure($method);
     }
+    return;
+}
 
-    # add passport if operation needs it
-    # this needs to be made smarter, probably by introspecting what operations
-    # inputs take a passport header
-    if ( any { $operation_name ne $_ }
-        qw(login ssoLogin mapSso logout searchMore searchNext) )
-    {
-        unshift @call_options, passport => $self->passport;
-    }
+sub _method_closure {
+    my $method = shift;
+    return sub {
+        my $self = shift;
+        $self->wsdl->compileCall( $method, transport => $self->transport );
+        my @results = $self->wsdl->call( $method => @_ );
+        if ( not $results[0] ) {
+            for ( $results[1]->errors ) { $_->throw }
+        }
+        return wantarray ? @results : $results[0];
+    };
+}
 
-    ## no critic (ValuesAndExpressions::ProhibitArrayAssignAref)
-    # make the call and either return or throw an exception
-    my @results = [ $self->_calls->{$operation_name}
-            ->( parameters => $parameters_ref, @call_options ) ];
-    my ( $answer_ref, $trace ) = @{ $results[-1] };
-    if ( not $answer_ref ) {
-        for ( $trace->errors ) { $_->throw }
-    }
+sub _around_closure {
+    my $method = shift;
+    return sub {
+        my ( $orig, $self ) = splice @_, 0, 2;
+        my ( $parameters_ref, @call_options )
+            = 'HASH' eq ref $_[0] ? @_ : {@_};
+
+        # add passport if operation needs it
+        # this needs to be made smarter, probably by introspecting what
+        # operations inputs take a passport header
+        if ( any { $method ne $_ }
+            qw(login ssoLogin mapSso logout searchMore searchNext) )
+        {
+            unshift @call_options, passport => $self->passport;
+        }
+        $self->_operation_name($method);
+        ## no critic (ValuesAndExpressions::ProhibitArrayAssignAref)
+        my @results = [
+            $self->$orig( { parameters => $parameters_ref, @call_options }, ),
+        ];
+
+        push @results =>
+            $self->_handle_exceeded_limits( $method, $parameters_ref,
+            $results[-1][0] );
+        return ( @results > 1 ? @results : @{ $results[0] } );
+    };
+}
+
+sub _handle_exceeded_limits {
+    my ( $self, $method, $parameters_ref, $answer_ref ) = @_;
 
     ## no critic (ValuesAndExpressions::ProhibitAccessOfPrivateData)
     if ( my $faults_ref = $answer_ref->{Fault} ) {
@@ -49,33 +73,17 @@ sub _call {
         if ((   any { $fault_name eq $_ }
                 qw(exceededRecordCountFault exceededRequestSizeFault)
             )
-            and $operation_name =~ / List $/xms
+            and $method =~ / List $/xms
             and exists $parameters_ref->{record}
             and 'ARRAY' eq ref $parameters_ref->{record}
             and @{ $parameters_ref->{record} } > 1
             )
         {
-            push @results => map { $self->_call( @{$_} ) }
+            return
+                map { $self->$method( @{$_} ) }
                 _halve_array( @{ $parameters_ref->{record} } );
         }
     }
-
-    return ( @results > 1 ? @results : @{ $results[0] } );
-}
-
-sub BUILD {
-    my $instance = shift;
-    for my $method ( map { $_->name } $instance->wsdl->operations ) {
-        ## no critic (ProhibitNoStrict,ProhibitNoWarnings)
-        no strict 'refs';
-        no warnings 'redefine';
-        *{$method} = sub {
-            my $self    = shift;
-            my @results = $self->_call(
-                $method => ( 'HASH' eq ref $_[0] ? $_[0] : {@_} ) );
-            return wantarray ? @results : $results[0];
-        };
-    }
     return;
 }
 
@@ -88,7 +96,7 @@ sub _halve_array {
 
 1;
 
-# ABSTRACT: NetSuite SOAP interface that caches compiled calls in memory
+# ABSTRACT: NetSuite SOAP interface as compiled Perl methods
 
 __END__
 
@@ -101,11 +109,11 @@ cpants kwalitee diff irc mailto metadata placeholders metacpan
 
 =head1 NAME
 
-WebService::NetSuite::SuiteTalk - NetSuite SOAP interface that caches compiled calls in memory
+WebService::NetSuite::SuiteTalk - NetSuite SOAP interface as compiled Perl methods
 
 =head1 VERSION
 
-version 0.006
+version 0.007
 
 =head1 SYNOPSIS
 
@@ -127,11 +135,11 @@ version 0.006
 
 =head1 DESCRIPTION
 
-This class provides an in-memory lazy cache for compiled SOAP calls using
+This class provides a Perl method API for
 L<NetSuite SuiteTalk|http://www.netsuite.com/portal/platform/developer/suitetalk.shtml>
-web services. The first call to any SOAP method uses
-L<XML::Compile::WSDL11|XML::Compile::WSDL11> to compile and execute that method
-against the specified NetSuite SuiteTalk service; subsequent calls can vary the
+web services. The first call to any SuiteTalk SOAP operation uses
+L<XML::Compile::WSDL11|XML::Compile::WSDL11> to compile and execute against the
+specified NetSuite SuiteTalk service; subsequent calls can vary the
 parameters but will use the same compiled code.
 
 =head1 METHODS
@@ -143,6 +151,11 @@ containing the results of the SOAP call; in list context they return the
 results hashref and an L<XML::Compile::SOAP::Trace|XML::Compile::SOAP::Trace>
 object suitable for debugging and exception handling.
 
+Note that all methods except C<login>, C<ssoLogin>, C<mapSso>, C<logout>,
+C<searchMore> and C<searchNext> are automatically passed request-level
+credentials as a C<passport> object in the SOAP header, thus eliminating the
+need for stateful session management.
+
 If there is no result then an exception will be thrown.
 
 =head2 Special note about list methods
@@ -459,7 +472,9 @@ Example:
 
 Because this class consumes the
 C<WebService::NetSuite::SuiteTalk::Role::Connection> role, it also has that
-role's attributes and methods. For example, you can use the C<user_agent>
+role's attributes and methods. Please consult that module for attributes
+important for establishing a connection such as C<account_id> and C<password>.
+You can also use other attributes, such as the C<user_agent>
 attribute to access the L<LWP::UserAgent|LWP::UserAgent> used to retrieve and
 call the SuiteTalk service, or the C<wsdl> method to access the underlying
 L<XML::Compile::WSDL11|XML::Compile::WSDL11> object.
